package jmednafen;

import java.io.*;
import java.util.ArrayList;

/**
 *
 * @author moises
 */
public class FileLoader
{
    private final static int MINIMUM_CONFIG_ENTRY_LENGTH = 8;
    private final static int MINIMUM_PLUGIN_ENTRY_LENGTH = 9;
    public final static int VIDEO = 1;
    public final static int AUDIO = 2;
    public final static int OTHER = 3;
    public final static int CONTROL = 4;
    public final static int GAME = 5;
    public final static int INGAME = 6;
    public final static int NETWORK = 7;
    
    public static ConfigRow createConfigRow ( int type )
    {
        ConfigRow tmp;
            
        switch ( type )
        {
            case VIDEO:
                        tmp = new ConfigRow ( ConfigRow.CHECKBOX , "-video.fs" );
                        tmp.setDefaultValue ( false );
                        tmp.setCurrentValue ( false );
                        tmp.setDescription ( "Enable fullscreen mode" );
                        break;
            case AUDIO:
                        tmp = new ConfigRow ( ConfigRow.CHECKBOX , "-sound" );
                        tmp.setDefaultValue ( true );
                        tmp.setCurrentValue ( true );
                        tmp.setDescription ( "Enable sound" );
                        break;
            case OTHER:
                        tmp = new ConfigRow ( ConfigRow.SLIDER , "-ffspeed" );
                        tmp.setValues ( 1 , 15 );
                        tmp.setDefaultValue ( 4 );
                        tmp.setCurrentValue ( 4 );
                        tmp.setDescription ( "Fast-forwarding speed multiplier" );
                        break;
            case CONTROL:
                            tmp = new ConfigRow ( ConfigRow.SLIDER , "-ffspeed" );
                            tmp.setValues ( 1 , 15 );
                            tmp.setDefaultValue ( 4 );
                            tmp.setCurrentValue ( 4 );
                            tmp.setDescription ( "Fast-forwarding speed multiplier" );
                            break;
            case GAME:
                        tmp = new ConfigRow ( ConfigRow.SLIDER , "-ffspeed" );
                        tmp.setValues ( 1 , 15 );
                        tmp.setDefaultValue ( 4 );
                        tmp.setCurrentValue ( 4 );
                        tmp.setDescription ( "Fast-forwarding speed multiplier" );
                        break;
            case INGAME:
                        tmp = new ConfigRow ( ConfigRow.SLIDER , "-ffspeed" );
                        tmp.setValues ( 1 , 15 );
                        tmp.setDefaultValue ( 4 );
                        tmp.setCurrentValue ( 4 );
                        tmp.setDescription ( "Fast-forwarding speed multiplier" );
                        break;
            case NETWORK:
                            tmp = new ConfigRow ( ConfigRow.SLIDER , "-ffspeed" );
                            tmp.setValues ( 1 , 15 );
                            tmp.setDefaultValue ( 4 );
                            tmp.setCurrentValue ( 4 );
                            tmp.setDescription ( "Fast-forwarding speed multiplier" );
                            break;
            default:
                        tmp = new ConfigRow ( ConfigRow.CHECKBOX , "-video.fs" );
                        tmp.setCurrentValue ( false );
                        tmp.setDefaultValue ( false );
                        tmp.setDescription ( "Enable fullscreen mode" );
                        break;
        }
        
        return ( tmp );
    }
    
    public static void loadStoredValues ( String file_path , ArrayList<ConfigRow> options , int type )
    {
        BufferedReader br;
        
        try
        {
            br = new BufferedReader ( new FileReader ( file_path + ".sav" ) );
        }
        catch ( FileNotFoundException e )
        {
            FileCreator.createStoredValuesFile ( file_path , options , FileCreator.CREATE );
            
            return;
        }
        
        String line_readed;
        
        int i = 0;
        
        do
        {
            try
            {
                line_readed = br.readLine ();
            }
            catch ( IOException e )
            {
                System.out.println ( "Error reading plugin values file. Returning default values instead." );
                return;
            }
            
            if ( line_readed != null )
            {
                if ( line_readed.length () >= 1 )
                {
                    switch ( options.get ( i ).getType () )
                    {
                        case ConfigRow.CHECKBOX:
                                                    if ( Integer.parseInt ( line_readed ) == 1 )
                                                    {
                                                        options.get ( i ).setCurrentValue ( true );
                                                    }
                                                    else
                                                    {
                                                        options.get ( i ).setCurrentValue ( false );
                                                    }
                                                    break;
                        case ConfigRow.CONDITIONAL_CHECKBOX:
                                                    if ( Integer.parseInt ( line_readed ) == 1 )
                                                    {
                                                        options.get ( i ).setCurrentValue ( true );
                                                    }
                                                    else
                                                    {
                                                        options.get ( i ).setCurrentValue ( false );
                                                    }
                                                    break;
                        case ConfigRow.COMBOBOX:
                                                    options.get ( i ).setCurrentValue ( line_readed );
                                                    break;
                        case ConfigRow.SLIDER:
                                                options.get ( i ).setCurrentValue( Integer.parseInt( line_readed ) );
                                                break;
                        case ConfigRow.TEXTFIELD:
                                                    options.get ( i ).setCurrentValue ( line_readed );
                                                    break;
                        case ConfigRow.SEPARATOR:
                                                    break;
                        case ConfigRow.TITLE:
                                                options.get ( i ).setCurrentValue ( line_readed );
                                                break;
                        default:    
                                    options.get ( i ).setCurrentValue ( line_readed );
                                    break;
                    }
                    i++;
                }
            }
        }
        while ( line_readed != null && i < options.size () );
    }
    
    public static OptionContainer loadPluginFile ( String file_path , int type )
    {
        OptionContainer container = new OptionContainer ();
        ArrayList<ConfigRow> list;
        
        list = new ArrayList<ConfigRow> ();
        
        File file = new File ( file_path );
        
        if ( !file.isFile () )
        {
            FileCreator.createPluginFile ( file_path , type );
        }
        
        BufferedReader br;
        
        try
        {
            br = new BufferedReader ( new FileReader ( file_path ) );
        }
        catch ( FileNotFoundException e )
        {
            System.out.println ( "Error opening plugin: " + file_path );
            System.out.println ( "   -> Returning fake plugin info." );
            
            container.setAuthor ( "Tuxman (tuxmansoft.blogspot.com)" );
            container.setVersion ( "2.0" );
            container.setDescription ( "Emergency default plugin for JMednafen." );
            
            ConfigRow tmp;
            
            tmp = createConfigRow ( type );
            
            list.add ( tmp );
            
            container.setRows ( list );
            
            return ( container );
        }
        
        String line_readed;
        
        do
        {
            try
            {
                line_readed = br.readLine ();
            }
            catch ( IOException e )
            {
                System.out.println ( "Error reading plugin file. The readed values will be skiped. Returning default values instead." );
                container.setAuthor ( "Tuxman (tuxmansoft.blogspot.com)" );
                container.setVersion ( "2.0" );
                container.setDescription ( "Emergency default plugin for JMednafen." );

                ConfigRow tmp;

                tmp = createConfigRow ( type );

                list.add ( tmp );
                
                container.setRows ( list );
                
                return ( container );
            }
            
            if ( line_readed != null )
            {
                if ( line_readed.length () >= FileLoader.MINIMUM_PLUGIN_ENTRY_LENGTH )
                {
                    String[] elements;
                    elements = line_readed.split ( "::" );
                    
                    if ( elements.length >= 2 )
                    {
                        for ( int i = 0; i < elements.length; i++ )
                        {
                            if ( elements[ i ].length () == 0 )
                            {
                                continue;
                            }
                        }
                        if ( elements[ 0 ].toUpperCase ().equals ( "AUTHOR" ) )
                        {
                           container.setAuthor ( elements[ 1 ] );
                        }
                        else if ( elements[ 0 ].toUpperCase ().equals ( "VERSION" ) )
                        {
                           container.setVersion ( elements[ 1 ] );
                        }
                        else if ( elements[ 0 ].toUpperCase ().equals ( "DESCRIPTION" ) )
                        {
                           container.setDescription ( elements[ 1 ] );
                        }
                        else if ( elements[ 0 ].toUpperCase ().equals ( "OPTION" ) )
                        {
                           ConfigRow tmp;
                           if ( elements.length >= 5 )
                           {
                               if ( elements[ 1 ].toUpperCase ().equals ( "TEXTFIELD" ) )
                               {
                                   tmp = new ConfigRow ( ConfigRow.TEXTFIELD , elements[ 2 ] );
                                   tmp.setDescription ( elements[ 3 ] );
                                   tmp.setDefaultValue ( elements[ 4 ] );
                                   list.add ( tmp );
                               }
                               else if ( elements[ 1 ].toUpperCase ().equals ( "CHECKBOX" ) )
                               {
                                   tmp = new ConfigRow ( ConfigRow.CHECKBOX , elements[ 2 ] );
                                   tmp.setDescription ( elements[ 3 ] );
                                   if ( Integer.parseInt ( elements[ 4 ] ) == 1 )
                                   {
                                       tmp.setDefaultValue ( true );
                                   }
                                   else
                                   {
                                       tmp.setDefaultValue ( false );
                                   }
                                   list.add ( tmp );
                               }
                               else if ( elements[ 1 ].toUpperCase ().equals ( "CONDITIONALCHECKBOX" ) )
                               {
                                   tmp = new ConfigRow ( ConfigRow.CONDITIONAL_CHECKBOX , elements[ 2 ] );
                                   tmp.setDescription ( elements[ 3 ] );
                                   if ( Integer.parseInt ( elements[ 4 ] ) == 1 )
                                   {
                                       tmp.setDefaultValue ( true );
                                   }
                                   else
                                   {
                                       tmp.setDefaultValue ( false );
                                   }
                                   list.add ( tmp );
                               }
                               else if ( elements[ 1 ].toUpperCase ().equals ( "SLIDER" ) )
                               {
                                   tmp = new ConfigRow ( ConfigRow.SLIDER , elements[ 2 ] );
                                   tmp.setDescription ( elements[ 3 ] );
                                   tmp.setDefaultValue ( Integer.parseInt ( elements[ 4 ] ) );
                                   String[] range;
                                   range = elements[ 5 ].split ( ":" );
                                   tmp.setValues ( Integer.parseInt ( range[ 0 ] ) , Integer.parseInt ( range[ 1 ] ) );
                                   list.add ( tmp );
                               }
                               else if ( elements[ 1 ].toUpperCase ().equals ( "COMBOBOX" ) )
                               {
                                   tmp = new ConfigRow ( ConfigRow.COMBOBOX , elements[ 2 ] );
                                   tmp.setDescription ( elements[ 3 ] );
                                   tmp.setDefaultValue ( elements[ 4 ] );
                                   
                                   String[] options_and_values;
                                   options_and_values = elements[ 5 ].split ( ":" );
                                   
                                   String[] options = new String [ options_and_values.length / 2 ];
                                   String[] values = new String [ options_and_values.length / 2 ];
                                   
                                   for ( int i = 0 , j = 0; i < options.length; i++ , j += 2 )
                                   {
                                       options[ i ] = options_and_values[ j ];
                                       values[ i ] = options_and_values[ j + 1 ];
                                   }
                                   
                                   tmp.setOptions ( options );
                                   tmp.setValues ( values );
                                   
                                   list.add ( tmp );
                               }
                           }
                           else if ( elements.length >= 2 )
                           {
                               if ( elements[ 1 ].toUpperCase ().equals ( "TITLE" ) )
                               {
                                   tmp = new ConfigRow ( ConfigRow.TITLE , "Dummy" );
                                   tmp.setDescription ( elements[ 2 ] );
                                   list.add ( tmp );
                               }
                               else if ( elements[ 1 ].toUpperCase ().equals ( "SEPARATOR" ) )
                               {
                                   tmp = new ConfigRow ( ConfigRow.SEPARATOR , "Dummy" );
                                   list.add ( tmp );
                               }
                           }
                        }
                    }
                }
            }
        }
        while ( line_readed != null );
        
        container.setRows ( list );
        
        return ( container );
    }
    
    public static InternalConfiguration loadInternalConfiguration ( String file_path )
    {
        InternalConfiguration config;
        config = new InternalConfiguration ();
        
        File file_file;
        file_file = new File ( file_path );
        
        if ( !file_file.isFile () )
        {
            FileCreator.createInternalConfiguration ( file_path , FileCreator.CREATE , new InternalConfiguration () );
        }
        
        BufferedReader br;
        
        try
        {
            br = new BufferedReader ( new FileReader ( file_path ) );
        }
        catch ( FileNotFoundException e )
        {
            System.out.println ( "Error trying to open internal configuration file." );
            return ( config );
        }
        
        String line_readed;
        
        do
        {
            try
            {
                line_readed = br.readLine ();
            }
            catch ( IOException e )
            {
                System.out.println ( "Error reading configuration file. The readed values will be skiped. Returning default values instead." );
                config = new InternalConfiguration ();
                return ( config );
            }
            
            if ( line_readed != null )
            {
                if ( line_readed.length () >= FileLoader.MINIMUM_CONFIG_ENTRY_LENGTH )
                {
                    String[] elements;
                    elements = line_readed.split ( "::" );
                    
                    if ( elements.length == 2 )
                    {
                        if ( elements[ 0 ].length () >= 4 && elements[ 1 ].length () >= 1 )
                        {
                            if ( elements[ 0 ].toUpperCase ().equals ( "AUDIO" ) )
                            {
                                config.setAudioPluginPath ( elements[ 1 ] );
                            }
                            else if ( elements[ 0 ].toUpperCase ().equals ( "VIDEO" ) )
                            {
                                config.setVideoPluginPath ( elements[ 1 ] );
                            }
                            else if ( elements[ 0 ].toUpperCase ().equals ( "OTHER" ) )
                            {
                                config.setOtherPluginPath ( elements[ 1 ] );
                            }
                            else if ( elements[ 0 ].toUpperCase ().equals ( "CONTROL" ) )
                            {
                                config.setControlPluginPath ( elements[ 1 ] );
                            }
                            else if ( elements[ 0 ].toUpperCase ().equals ( "INGAME" ) )
                            {
                                config.setInGamePluginPath ( elements[ 1 ] );
                            }
                            else if ( elements[ 0 ].toUpperCase ().equals ( "NETWORK" ) )
                            {
                                config.setNetworkPluginPath ( elements[ 1 ] );
                            }
                            else if ( elements[ 0 ].toUpperCase ().equals ( "MEDNAFENPATH" ) )
                            {
                                config.setMednafenPath ( elements[ 1 ] );
                            }
                            else if ( elements[ 0 ].toUpperCase ().equals ( "EXTENTIONS" ) )
                            {
                                config.setExtentions ( elements[ 1 ].split ( ":" ) );
                            }
                            else if ( elements[ 0 ].toUpperCase ().equals ( "COMMANDFORMAT" ) )
                            {
                                config.setCommandFormat ( Integer.parseInt ( elements[ 1 ] ) );
                            }
                        }
                    }
                }
            }
        }
        while ( line_readed != null );
        
        return ( config );
    }
}
