package jmednafen;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import javax.swing.JOptionPane;

/**
 *
 * @author moises
 */
public class PluginCreator implements PluginVersions , PluginNames
{
    private static String getVideoPluginName ( int mednafen_version )
    {
        String name = new String ();
        
        switch ( mednafen_version )
        {
            case MEDNAFEN_0_9_21:
                name = VIDEO_0_9_21;
                break;
            case MEDNAFEN_0_8_D:
                name = VIDEO_0_8_D;
                break;
        }
        
        return ( name );
    }
    
    private static String getAudioPluginName ( int mednafen_version )
    {
        String name = new String ();
        
        switch ( mednafen_version )
        {
            case MEDNAFEN_0_9_21:
                name = AUDIO_0_9_21;
                break;
            case MEDNAFEN_0_8_D:
                name = AUDIO_0_8_D;
                break;
        }
        
        return ( name );
    }
    
    private static String getControlsPluginName ( int mednafen_version )
    {
        String name = new String ();
        
        switch ( mednafen_version )
        {
            case MEDNAFEN_0_9_21:
                name = CONTROLS_0_9_21;
                break;
            case MEDNAFEN_0_8_D:
                name = CONTROLS_0_8_D;
                break;
        }
        
        return ( name );
    }
    
    private static String getInGamePluginName ( int mednafen_version )
    {
        String name = new String ();
        
        switch ( mednafen_version )
        {
            case MEDNAFEN_0_9_21:
                name = INGAME_0_9_21;
                break;
            case MEDNAFEN_0_8_D:
                name = INGAME_0_8_D;
                break;
        }
        
        return ( name );
    }
    
    private static String getNetworkPluginName ( int mednafen_version )
    {
        String name = new String ();
        
        switch ( mednafen_version )
        {
            case MEDNAFEN_0_9_21:
                name = NETWORK_0_9_21;
                break;
            case MEDNAFEN_0_8_D:
                name = NETWORK_0_8_D;
                break;
        }
        
        return ( name );
    }
    
    private static String getOtherPluginName ( int mednafen_version )
    {
        String name = new String ();
        
        switch ( mednafen_version )
        {
            case MEDNAFEN_0_9_21:
                name = OTHER_0_9_21;
                break;
            case MEDNAFEN_0_8_D:
                name = OTHER_0_8_D;
                break;
        }
        
        return ( name );
    }
    
    private static void printPreamble ( PrintWriter output_file )
    {
        output_file.println ( "Author::Tuxman (tuxmansoft.blogspot.com)" );
        output_file.println ( "Version::2.0" );
        output_file.println ( "Description::Default plugin for JMednafen. Support of older Mednafen versions is available using other plugins (change the plugins in the config window)." );
    }
    
    public static boolean makeVideoPlugin ( String folder_path , int mednafen_version )
    {
        PrintWriter output_file;
        
        File file = new File ( folder_path );
        
        if ( file.isFile () )
        {
            if ( !file.delete () )
            {
                JOptionPane.showMessageDialog ( null , "Error deleing old plugin file on: " + folder_path + "." , "Error creating plugin file.", JOptionPane.ERROR_MESSAGE );
                return ( false );
            }
        }
        
        try
        {
            output_file = new PrintWriter ( new FileWriter ( folder_path + getVideoPluginName ( mednafen_version ) ) );
        }
        catch ( IOException e )
        {
            JOptionPane.showMessageDialog ( null , "Error creating plugin file: " + getVideoPluginName ( mednafen_version ) + "." , "Error creating plugin file", JOptionPane.ERROR_MESSAGE );
            return ( false );
        }
       
        printPreamble ( output_file );
        
        switch ( mednafen_version )
        {
            case MEDNAFEN_0_9_21:
                output_file.println ( "Option::Title::Generic options (all systems)" );
                output_file.println ( "Option::Separator" );
                output_file.println ( "Option::Checkbox::-video.fs::Enable fullscreen mode::0" );
                output_file.println ( "Option::Combobox::-video.driver::Select video driver::OpenGL::OpenGL:opengl:Simple Directmedia Layer:sdl:Overlay:overlay" );
                output_file.println ( "Option::Checkbox::-video.glvsync::Attempt to synchronize OpenGL page flips to vertical retrace period.::1" );
                output_file.println ( "Option::Checkbox::-video.frameskip::Enable frameskip during emulation rendering.::1" );
                
                output_file.println ( "Option::Separator" );
                output_file.println ( "Option::Title::Sony PlayStation" );
                output_file.println ( "Option::Separator" );
                output_file.println ( "Option::Combobox::-psx.pixshader::Enable specified OpenGL pixel shader.::None::None:none:Auto Interpolation:autoip:Sharper Auto Interpolation:autoipsharper:Scale2x:scale2x:Sharper bilinear interpolation.:ipsharper:Linear interpolation on X axis only.:ipxnoty:Linear interpolation on Y axis only.:ipynotx:Sharper version of ipxnoty.:ipxnotysharper:Sharper version of ipynotx.:ipynotxsharper" );
                output_file.println ( "Option::Slider::-psx.scanlines::Enable scanlines with specified opacity.::0::0:100" );
                output_file.println ( "Option::Combobox::-psx.special::Enable specified special video scaler.::None::None:none:HQ2X Layer:hq2x:HQ3X:hq3x:HQ4X:hq4x:Scale 2X:scale2x:Scale 3X:scale3x:Scale 4X:scale4x:2XSAI:2xsai:Super 2XSAI:super2xsai:Super Eagle:supereagle:Nearest-neighbor 2X:nn2x:Nearest-Neighbor 3X:nn3x:Nearest-neighbor 4X:nn4x:Nearest-neighbor 2X, y axis only:nny2x:Nearest-neighbor 3X, y axis only:nny3x:Nearest-neighbor 4X, y axis only:nny4x" );
                output_file.println ( "Option::Combobox::-psx.stretch::Stretch to fill screen::Aspect Preserve + Integer Multiple-of-2 Scale::Disabled:0:Full:full:Aspect Preserve:aspect:Aspect Preserve + Integer Scale:aspect_int:Aspect Preserve + Integer Multiple-of-2 Scale:aspect_mult2" );
                
                output_file.println ( "Option::Separator" );
                output_file.println ( "Option::Title::Nintendo Entertainment System/Famicom" );
                output_file.println ( "Option::Separator" );
                output_file.println ( "Option::Slider::-nes.scanlines::Enable scanlines with specified opacity.::0::0:100" );
                output_file.println ( "Option::Combobox::-nes.special::Enable specified special video scaler.::None::None:none:HQ2X Layer:hq2x:HQ3X:hq3x:HQ4X:hq4x:Scale 2X:scale2x:Scale 3X:scale3x:Scale 4X:scale4x:2XSAI:2xsai:Super 2XSAI:super2xsai:Super Eagle:supereagle:Nearest-neighbor 2X:nn2x:Nearest-Neighbor 3X:nn3x:Nearest-neighbor 4X:nn4x:Nearest-neighbor 2X, y axis only:nny2x:Nearest-neighbor 3X, y axis only:nny3x:Nearest-neighbor 4X, y axis only:nny4x" );
                
                output_file.println ( "Option::Separator" );
                output_file.println ( "Option::Title::Super Nintendo Entertainment System/Super Famicom" );
                output_file.println ( "Option::Separator" );
                output_file.println ( "Option::Combobox::-snes.pixshader::Enable specified OpenGL pixel shader.::None::None:none:Auto Interpolation:autoip:Sharper Auto Interpolation:autoipsharper:Scale2x:scale2x:Sharper bilinear interpolation.:ipsharper:Linear interpolation on X axis only.:ipxnoty:Linear interpolation on Y axis only.:ipynotx:Sharper version of ipxnoty.:ipxnotysharper:Sharper version of ipynotx.:ipynotxsharper" );
                output_file.println ( "Option::Slider::-snes.scanlines::Enable scanlines with specified opacity.::0::0:100" );
                output_file.println ( "Option::Combobox::-snes.special::Enable specified special video scaler.::None::None:none:HQ2X Layer:hq2x:HQ3X:hq3x:HQ4X:hq4x:Scale 2X:scale2x:Scale 3X:scale3x:Scale 4X:scale4x:2XSAI:2xsai:Super 2XSAI:super2xsai:Super Eagle:supereagle:Nearest-neighbor 2X:nn2x:Nearest-Neighbor 3X:nn3x:Nearest-neighbor 4X:nn4x:Nearest-neighbor 2X, y axis only:nny2x:Nearest-neighbor 3X, y axis only:nny3x:Nearest-neighbor 4X, y axis only:nny4x" );
                
                output_file.println ( "Option::Separator" );
                output_file.println ( "Option::Title::Nintendo Game Boy (Color)" );
                output_file.println ( "Option::Separator" );
                output_file.println ( "Option::Combobox::-gb.pixshader::Enable specified OpenGL pixel shader.::None::None:none:Auto Interpolation:autoip:Sharper Auto Interpolation:autoipsharper:Scale2x:scale2x:Sharper bilinear interpolation.:ipsharper:Linear interpolation on X axis only.:ipxnoty:Linear interpolation on Y axis only.:ipynotx:Sharper version of ipxnoty.:ipxnotysharper:Sharper version of ipynotx.:ipynotxsharper" );
                output_file.println ( "Option::Slider::-gb.scanlines::Enable scanlines with specified opacity.::0::0:100" );
                output_file.println ( "Option::Combobox::-gb.special::Enable specified special video scaler.::None::None:none:HQ2X Layer:hq2x:HQ3X:hq3x:HQ4X:hq4x:Scale 2X:scale2x:Scale 3X:scale3x:Scale 4X:scale4x:2XSAI:2xsai:Super 2XSAI:super2xsai:Super Eagle:supereagle:Nearest-neighbor 2X:nn2x:Nearest-Neighbor 3X:nn3x:Nearest-neighbor 4X:nn4x:Nearest-neighbor 2X, y axis only:nny2x:Nearest-neighbor 3X, y axis only:nny3x:Nearest-neighbor 4X, y axis only:nny4x" );
                
                output_file.println ( "Option::Separator" );
                output_file.println ( "Option::Title::Nintendo Game Boy Advance" );
                output_file.println ( "Option::Separator" );
                output_file.println ( "Option::Combobox::-gba.pixshader::Enable specified OpenGL pixel shader.::None::None:none:Auto Interpolation:autoip:Sharper Auto Interpolation:autoipsharper:Scale2x:scale2x:Sharper bilinear interpolation.:ipsharper:Linear interpolation on X axis only.:ipxnoty:Linear interpolation on Y axis only.:ipynotx:Sharper version of ipxnoty.:ipxnotysharper:Sharper version of ipynotx.:ipynotxsharper" );
                output_file.println ( "Option::Slider::-gba.scanlines::Enable scanlines with specified opacity.::0::0:100" );
                output_file.println ( "Option::Combobox::-gba.special::Enable specified special video scaler.::None::None:none:HQ2X Layer:hq2x:HQ3X:hq3x:HQ4X:hq4x:Scale 2X:scale2x:Scale 3X:scale3x:Scale 4X:scale4x:2XSAI:2xsai:Super 2XSAI:super2xsai:Super Eagle:supereagle:Nearest-neighbor 2X:nn2x:Nearest-Neighbor 3X:nn3x:Nearest-neighbor 4X:nn4x:Nearest-neighbor 2X, y axis only:nny2x:Nearest-neighbor 3X, y axis only:nny3x:Nearest-neighbor 4X, y axis only:nny4x" );
                
                output_file.println ( "Option::Separator" );
                output_file.println ( "Option::Title::Nintendo Virtual Boy" );
                output_file.println ( "Option::Separator" );
                output_file.println ( "Option::Combobox::-vb.pixshader::Enable specified OpenGL pixel shader.::None::None:none:Auto Interpolation:autoip:Sharper Auto Interpolation:autoipsharper:Scale2x:scale2x:Sharper bilinear interpolation.:ipsharper:Linear interpolation on X axis only.:ipxnoty:Linear interpolation on Y axis only.:ipynotx:Sharper version of ipxnoty.:ipxnotysharper:Sharper version of ipynotx.:ipynotxsharper" );
                output_file.println ( "Option::Slider::-vb.scanlines::Enable scanlines with specified opacity.::0::0:100" );
                output_file.println ( "Option::Combobox::-vb.special::Enable specified special video scaler.::None::None:none:HQ2X Layer:hq2x:HQ3X:hq3x:HQ4X:hq4x:Scale 2X:scale2x:Scale 3X:scale3x:Scale 4X:scale4x:2XSAI:2xsai:Super 2XSAI:super2xsai:Super Eagle:supereagle:Nearest-neighbor 2X:nn2x:Nearest-Neighbor 3X:nn3x:Nearest-neighbor 4X:nn4x:Nearest-neighbor 2X, y axis only:nny2x:Nearest-neighbor 3X, y axis only:nny3x:Nearest-neighbor 4X, y axis only:nny4x" );
                
                output_file.println ( "Option::Separator" );
                output_file.println ( "Option::Title::Sega Game Gear" );
                output_file.println ( "Option::Separator" );
                output_file.println ( "Option::Combobox::-gg.pixshader::Enable specified OpenGL pixel shader.::None::None:none:Auto Interpolation:autoip:Sharper Auto Interpolation:autoipsharper:Scale2x:scale2x:Sharper bilinear interpolation.:ipsharper:Linear interpolation on X axis only.:ipxnoty:Linear interpolation on Y axis only.:ipynotx:Sharper version of ipxnoty.:ipxnotysharper:Sharper version of ipynotx.:ipynotxsharper" );
                output_file.println ( "Option::Slider::-gg.scanlines::Enable scanlines with specified opacity.::0::0:100" );
                output_file.println ( "Option::Combobox::-gg.special::Enable specified special video scaler.::None::None:none:HQ2X Layer:hq2x:HQ3X:hq3x:HQ4X:hq4x:Scale 2X:scale2x:Scale 3X:scale3x:Scale 4X:scale4x:2XSAI:2xsai:Super 2XSAI:super2xsai:Super Eagle:supereagle:Nearest-neighbor 2X:nn2x:Nearest-Neighbor 3X:nn3x:Nearest-neighbor 4X:nn4x:Nearest-neighbor 2X, y axis only:nny2x:Nearest-neighbor 3X, y axis only:nny3x:Nearest-neighbor 4X, y axis only:nny4x" );
                
                output_file.println ( "Option::Separator" );
                output_file.println ( "Option::Title::Sega Genesis/MegaDrive" );
                output_file.println ( "Option::Separator" );
                output_file.println ( "Option::Combobox::-md.pixshader::Enable specified OpenGL pixel shader.::None::None:none:Auto Interpolation:autoip:Sharper Auto Interpolation:autoipsharper:Scale2x:scale2x:Sharper bilinear interpolation.:ipsharper:Linear interpolation on X axis only.:ipxnoty:Linear interpolation on Y axis only.:ipynotx:Sharper version of ipxnoty.:ipxnotysharper:Sharper version of ipynotx.:ipynotxsharper" );
                output_file.println ( "Option::Slider::-md.scanlines::Enable scanlines with specified opacity.::0::0:100" );
                output_file.println ( "Option::Combobox::-md.special::Enable specified special video scaler.::None::None:none:HQ2X Layer:hq2x:HQ3X:hq3x:HQ4X:hq4x:Scale 2X:scale2x:Scale 3X:scale3x:Scale 4X:scale4x:2XSAI:2xsai:Super 2XSAI:super2xsai:Super Eagle:supereagle:Nearest-neighbor 2X:nn2x:Nearest-Neighbor 3X:nn3x:Nearest-neighbor 4X:nn4x:Nearest-neighbor 2X, y axis only:nny2x:Nearest-neighbor 3X, y axis only:nny3x:Nearest-neighbor 4X, y axis only:nny4x" );
                
                output_file.println ( "Option::Separator" );
                output_file.println ( "Option::Title::Sega Master System" );
                output_file.println ( "Option::Separator" );
                output_file.println ( "Option::Combobox::-sms.pixshader::Enable specified OpenGL pixel shader.::None::None:none:Auto Interpolation:autoip:Sharper Auto Interpolation:autoipsharper:Scale2x:scale2x:Sharper bilinear interpolation.:ipsharper:Linear interpolation on X axis only.:ipxnoty:Linear interpolation on Y axis only.:ipynotx:Sharper version of ipxnoty.:ipxnotysharper:Sharper version of ipynotx.:ipynotxsharper" );
                output_file.println ( "Option::Slider::-sms.scanlines::Enable scanlines with specified opacity.::0::0:100" );
                output_file.println ( "Option::Combobox::-sms.special::Enable specified special video scaler.::None::None:none:HQ2X Layer:hq2x:HQ3X:hq3x:HQ4X:hq4x:Scale 2X:scale2x:Scale 3X:scale3x:Scale 4X:scale4x:2XSAI:2xsai:Super 2XSAI:super2xsai:Super Eagle:supereagle:Nearest-neighbor 2X:nn2x:Nearest-Neighbor 3X:nn3x:Nearest-neighbor 4X:nn4x:Nearest-neighbor 2X, y axis only:nny2x:Nearest-neighbor 3X, y axis only:nny3x:Nearest-neighbor 4X, y axis only:nny4x" );
                
                output_file.println ( "Option::Separator" );
                output_file.println ( "Option::Title::SNK Neo Geo Pocket (Color)" );
                output_file.println ( "Option::Separator" );
                output_file.println ( "Option::Combobox::-ngp.pixshader::Enable specified OpenGL pixel shader.::None::None:none:Auto Interpolation:autoip:Sharper Auto Interpolation:autoipsharper:Scale2x:scale2x:Sharper bilinear interpolation.:ipsharper:Linear interpolation on X axis only.:ipxnoty:Linear interpolation on Y axis only.:ipynotx:Sharper version of ipxnoty.:ipxnotysharper:Sharper version of ipynotx.:ipynotxsharper" );
                output_file.println ( "Option::Slider::-ngp.scanlines::Enable scanlines with specified opacity.::0::0:100" );
                output_file.println ( "Option::Combobox::-ngp.special::Enable specified special video scaler.::None::None:none:HQ2X Layer:hq2x:HQ3X:hq3x:HQ4X:hq4x:Scale 2X:scale2x:Scale 3X:scale3x:Scale 4X:scale4x:2XSAI:2xsai:Super 2XSAI:super2xsai:Super Eagle:supereagle:Nearest-neighbor 2X:nn2x:Nearest-Neighbor 3X:nn3x:Nearest-neighbor 4X:nn4x:Nearest-neighbor 2X, y axis only:nny2x:Nearest-neighbor 3X, y axis only:nny3x:Nearest-neighbor 4X, y axis only:nny4x" );
                
                output_file.println ( "Option::Separator" );
                output_file.println ( "Option::Title::NEC PC Engine (CD)/TurboGrafx 16 (CD)/SuperGrafx" );
                output_file.println ( "Option::Separator" );
                output_file.println ( "Option::Combobox::-pce.pixshader::Enable specified OpenGL pixel shader.::None::None:none:Auto Interpolation:autoip:Sharper Auto Interpolation:autoipsharper:Scale2x:scale2x:Sharper bilinear interpolation.:ipsharper:Linear interpolation on X axis only.:ipxnoty:Linear interpolation on Y axis only.:ipynotx:Sharper version of ipxnoty.:ipxnotysharper:Sharper version of ipynotx.:ipynotxsharper" );
                output_file.println ( "Option::Slider::-pce.scanlines::Enable scanlines with specified opacity.::0::0:100" );
                output_file.println ( "Option::Combobox::-pce.special::Enable specified special video scaler.::None::None:none:HQ2X Layer:hq2x:HQ3X:hq3x:HQ4X:hq4x:Scale 2X:scale2x:Scale 3X:scale3x:Scale 4X:scale4x:2XSAI:2xsai:Super 2XSAI:super2xsai:Super Eagle:supereagle:Nearest-neighbor 2X:nn2x:Nearest-Neighbor 3X:nn3x:Nearest-neighbor 4X:nn4x:Nearest-neighbor 2X, y axis only:nny2x:Nearest-neighbor 3X, y axis only:nny3x:Nearest-neighbor 4X, y axis only:nny4x" );
                
                output_file.println ( "Option::Separator" );
                output_file.println ( "Option::Title::NEC PC Engine (CD)/TurboGrafx 16 (CD)/SuperGrafx -> Fast emulation" );
                output_file.println ( "Option::Separator" );
                output_file.println ( "Option::Combobox::-pce_fast.pixshader::Enable specified OpenGL pixel shader.::None::None:none:Auto Interpolation:autoip:Sharper Auto Interpolation:autoipsharper:Scale2x:scale2x:Sharper bilinear interpolation.:ipsharper:Linear interpolation on X axis only.:ipxnoty:Linear interpolation on Y axis only.:ipynotx:Sharper version of ipxnoty.:ipxnotysharper:Sharper version of ipynotx.:ipynotxsharper" );
                output_file.println ( "Option::Slider::-pce_fast.scanlines::Enable scanlines with specified opacity.::0::0:100" );
                output_file.println ( "Option::Combobox::-pce_fast.special::Enable specified special video scaler.::None::None:none:HQ2X Layer:hq2x:HQ3X:hq3x:HQ4X:hq4x:Scale 2X:scale2x:Scale 3X:scale3x:Scale 4X:scale4x:2XSAI:2xsai:Super 2XSAI:super2xsai:Super Eagle:supereagle:Nearest-neighbor 2X:nn2x:Nearest-Neighbor 3X:nn3x:Nearest-neighbor 4X:nn4x:Nearest-neighbor 2X, y axis only:nny2x:Nearest-neighbor 3X, y axis only:nny3x:Nearest-neighbor 4X, y axis only:nny4x" );
                
                output_file.println ( "Option::Separator" );
                output_file.println ( "Option::Title::NEC PC-FX" );
                output_file.println ( "Option::Separator" );
                output_file.println ( "Option::Combobox::-pcfx.pixshader::Enable specified OpenGL pixel shader.::None::None:none:Auto Interpolation:autoip:Sharper Auto Interpolation:autoipsharper:Scale2x:scale2x:Sharper bilinear interpolation.:ipsharper:Linear interpolation on X axis only.:ipxnoty:Linear interpolation on Y axis only.:ipynotx:Sharper version of ipxnoty.:ipxnotysharper:Sharper version of ipynotx.:ipynotxsharper" );
                output_file.println ( "Option::Slider::-pcfx.scanlines::Enable scanlines with specified opacity.::0::0:100" );
                output_file.println ( "Option::Combobox::-pcfx.special::Enable specified special video scaler.::None::None:none:HQ2X Layer:hq2x:HQ3X:hq3x:HQ4X:hq4x:Scale 2X:scale2x:Scale 3X:scale3x:Scale 4X:scale4x:2XSAI:2xsai:Super 2XSAI:super2xsai:Super Eagle:supereagle:Nearest-neighbor 2X:nn2x:Nearest-Neighbor 3X:nn3x:Nearest-neighbor 4X:nn4x:Nearest-neighbor 2X, y axis only:nny2x:Nearest-neighbor 3X, y axis only:nny3x:Nearest-neighbor 4X, y axis only:nny4x" );
                
                output_file.println ( "Option::Separator" );
                output_file.println ( "Option::Title::Bandai Wonder Swan" );
                output_file.println ( "Option::Separator" );
                output_file.println ( "Option::Combobox::-wswan.pixshader::Enable specified OpenGL pixel shader.::None::None:none:Auto Interpolation:autoip:Sharper Auto Interpolation:autoipsharper:Scale2x:scale2x:Sharper bilinear interpolation.:ipsharper:Linear interpolation on X axis only.:ipxnoty:Linear interpolation on Y axis only.:ipynotx:Sharper version of ipxnoty.:ipxnotysharper:Sharper version of ipynotx.:ipynotxsharper" );
                output_file.println ( "Option::Slider::-wswan.scanlines::Enable scanlines with specified opacity.::0::0:100" );
                output_file.println ( "Option::Combobox::-wswan.special::Enable specified special video scaler.::None::None:none:HQ2X Layer:hq2x:HQ3X:hq3x:HQ4X:hq4x:Scale 2X:scale2x:Scale 3X:scale3x:Scale 4X:scale4x:2XSAI:2xsai:Super 2XSAI:super2xsai:Super Eagle:supereagle:Nearest-neighbor 2X:nn2x:Nearest-Neighbor 3X:nn3x:Nearest-neighbor 4X:nn4x:Nearest-neighbor 2X, y axis only:nny2x:Nearest-neighbor 3X, y axis only:nny3x:Nearest-neighbor 4X, y axis only:nny4x" );
                
                output_file.println ( "Option::Separator" );
                output_file.println ( "Option::Title::Atari Lynx" );
                output_file.println ( "Option::Separator" );
                output_file.println ( "Option::Slider::-lynx.scanlines::Enable scanlines with specified opacity.::0::0:100" );
                output_file.println ( "Option::Combobox::-lynx.special::Enable specified special video scaler.::None::None:none:HQ2X Layer:hq2x:HQ3X:hq3x:HQ4X:hq4x:Scale 2X:scale2x:Scale 3X:scale3x:Scale 4X:scale4x:2XSAI:2xsai:Super 2XSAI:super2xsai:Super Eagle:supereagle:Nearest-neighbor 2X:nn2x:Nearest-Neighbor 3X:nn3x:Nearest-neighbor 4X:nn4x:Nearest-neighbor 2X, y axis only:nny2x:Nearest-neighbor 3X, y axis only:nny3x:Nearest-neighbor 4X, y axis only:nny4x" );
                break;
            case MEDNAFEN_0_8_D:
                output_file.println ( "Option::Title::Generic options (all systems)" );
                output_file.println ( "Option::Separator" );
                output_file.println ( "Option::Checkbox::-fs::Enable fullscreen mode::0" );
                output_file.println ( "Option::Combobox::-vdriver::Select video driver::OpenGL + SDL::OpenGL + SDL:opengl:SDL framebuffer:sdl" );
                output_file.println ( "Option::Checkbox::-glvsync::Attempt to synchronize OpenGL page flips to vertical retrace period.::1" );
                
                output_file.println ( "Option::Separator" );
                output_file.println ( "Option::Title::Nintendo Entertainment System/Famicom" );
                output_file.println ( "Option::Separator" );
                output_file.println ( "Option::Combobox::-nes.pixshader::Enable specified OpenGL pixel shader.::None::None:none:Auto Interpolation:autoip:Sharper Auto Interpolation:autoipsharper:Scale2x:scale2x:Sharper bilinear interpolation.:ipsharper:Linear interpolation on X axis only.:ipxnoty:Linear interpolation on Y axis only.:ipynotx:Sharper version of ipxnoty.:ipxnotysharper:Sharper version of ipynotx.:ipynotxsharper" );
                output_file.println ( "Option::Slider::-nes.scanlines::Enable scanlines with specified opacity.::0::0:100" );
                output_file.println ( "Option::Combobox::-nes.special::Enable specified special video scaler.::None::None:none:HQ2X Layer:hq2x:HQ3X:hq3x:HQ4X:hq4x:Scale 2X:scale2x:Scale 3X:scale3x:Scale 4X:scale4x:2XSAI:2xsai:Super 2XSAI:super2xsai:Super Eagle:supereagle:Nearest-neighbor 2X:nn2x:Nearest-Neighbor 3X:nn3x:Nearest-neighbor 4X:nn4x:Nearest-neighbor 2X, y axis only:nny2x:Nearest-neighbor 3X, y axis only:nny3x:Nearest-neighbor 4X, y axis only:nny4x" );
                output_file.println ( "Option::Checkbox::-nes.stretch::Stretch to fill screen when in fullscreen mode.::0" );
                
                output_file.println ( "Option::Separator" );
                output_file.println ( "Option::Title::Nintendo Game Boy (Color)" );
                output_file.println ( "Option::Separator" );
                output_file.println ( "Option::Combobox::-gb.pixshader::Enable specified OpenGL pixel shader.::None::None:none:Auto Interpolation:autoip:Sharper Auto Interpolation:autoipsharper:Scale2x:scale2x:Sharper bilinear interpolation.:ipsharper:Linear interpolation on X axis only.:ipxnoty:Linear interpolation on Y axis only.:ipynotx:Sharper version of ipxnoty.:ipxnotysharper:Sharper version of ipynotx.:ipynotxsharper" );
                output_file.println ( "Option::Slider::-gb.scanlines::Enable scanlines with specified opacity.::0::0:100" );
                output_file.println ( "Option::Combobox::-gb.special::Enable specified special video scaler.::None::None:none:HQ2X Layer:hq2x:HQ3X:hq3x:HQ4X:hq4x:Scale 2X:scale2x:Scale 3X:scale3x:Scale 4X:scale4x:2XSAI:2xsai:Super 2XSAI:super2xsai:Super Eagle:supereagle:Nearest-neighbor 2X:nn2x:Nearest-Neighbor 3X:nn3x:Nearest-neighbor 4X:nn4x:Nearest-neighbor 2X, y axis only:nny2x:Nearest-neighbor 3X, y axis only:nny3x:Nearest-neighbor 4X, y axis only:nny4x" );
                output_file.println ( "Option::Checkbox::-gb.stretch::Stretch to fill screen when in fullscreen mode.::0" );
                
                output_file.println ( "Option::Separator" );
                output_file.println ( "Option::Title::Nintendo Game Boy Advance" );
                output_file.println ( "Option::Separator" );
                output_file.println ( "Option::Combobox::-gba.pixshader::Enable specified OpenGL pixel shader.::None::None:none:Auto Interpolation:autoip:Sharper Auto Interpolation:autoipsharper:Scale2x:scale2x:Sharper bilinear interpolation.:ipsharper:Linear interpolation on X axis only.:ipxnoty:Linear interpolation on Y axis only.:ipynotx:Sharper version of ipxnoty.:ipxnotysharper:Sharper version of ipynotx.:ipynotxsharper" );
                output_file.println ( "Option::Slider::-gba.scanlines::Enable scanlines with specified opacity.::0::0:100" );
                output_file.println ( "Option::Combobox::-gba.special::Enable specified special video scaler.::None::None:none:HQ2X Layer:hq2x:HQ3X:hq3x:HQ4X:hq4x:Scale 2X:scale2x:Scale 3X:scale3x:Scale 4X:scale4x:2XSAI:2xsai:Super 2XSAI:super2xsai:Super Eagle:supereagle:Nearest-neighbor 2X:nn2x:Nearest-Neighbor 3X:nn3x:Nearest-neighbor 4X:nn4x:Nearest-neighbor 2X, y axis only:nny2x:Nearest-neighbor 3X, y axis only:nny3x:Nearest-neighbor 4X, y axis only:nny4x" );
                output_file.println ( "Option::Checkbox::-gba.stretch::Stretch to fill screen when in fullscreen mode.::0" );
                
                output_file.println ( "Option::Separator" );
                output_file.println ( "Option::Title::SNK Neo Geo Pocket (Color)" );
                output_file.println ( "Option::Separator" );
                output_file.println ( "Option::Combobox::-ngp.pixshader::Enable specified OpenGL pixel shader.::None::None:none:Auto Interpolation:autoip:Sharper Auto Interpolation:autoipsharper:Scale2x:scale2x:Sharper bilinear interpolation.:ipsharper:Linear interpolation on X axis only.:ipxnoty:Linear interpolation on Y axis only.:ipynotx:Sharper version of ipxnoty.:ipxnotysharper:Sharper version of ipynotx.:ipynotxsharper" );
                output_file.println ( "Option::Slider::-ngp.scanlines::Enable scanlines with specified opacity.::0::0:100" );
                output_file.println ( "Option::Combobox::-ngp.special::Enable specified special video scaler.::None::None:none:HQ2X Layer:hq2x:HQ3X:hq3x:HQ4X:hq4x:Scale 2X:scale2x:Scale 3X:scale3x:Scale 4X:scale4x:2XSAI:2xsai:Super 2XSAI:super2xsai:Super Eagle:supereagle:Nearest-neighbor 2X:nn2x:Nearest-Neighbor 3X:nn3x:Nearest-neighbor 4X:nn4x:Nearest-neighbor 2X, y axis only:nny2x:Nearest-neighbor 3X, y axis only:nny3x:Nearest-neighbor 4X, y axis only:nny4x" );
                output_file.println ( "Option::Checkbox::-ngp.stretch::Stretch to fill screen when in fullscreen mode.::0" );
                
                output_file.println ( "Option::Separator" );
                output_file.println ( "Option::Title::NEC PC Engine (CD)/TurboGrafx 16 (CD)/SuperGrafx" );
                output_file.println ( "Option::Separator" );
                output_file.println ( "Option::Combobox::-pce.pixshader::Enable specified OpenGL pixel shader.::None::None:none:Auto Interpolation:autoip:Sharper Auto Interpolation:autoipsharper:Scale2x:scale2x:Sharper bilinear interpolation.:ipsharper:Linear interpolation on X axis only.:ipxnoty:Linear interpolation on Y axis only.:ipynotx:Sharper version of ipxnoty.:ipxnotysharper:Sharper version of ipynotx.:ipynotxsharper" );
                output_file.println ( "Option::Slider::-pce.scanlines::Enable scanlines with specified opacity.::0::0:100" );
                output_file.println ( "Option::Combobox::-pce.special::Enable specified special video scaler.::None::None:none:HQ2X Layer:hq2x:HQ3X:hq3x:HQ4X:hq4x:Scale 2X:scale2x:Scale 3X:scale3x:Scale 4X:scale4x:2XSAI:2xsai:Super 2XSAI:super2xsai:Super Eagle:supereagle:Nearest-neighbor 2X:nn2x:Nearest-Neighbor 3X:nn3x:Nearest-neighbor 4X:nn4x:Nearest-neighbor 2X, y axis only:nny2x:Nearest-neighbor 3X, y axis only:nny3x:Nearest-neighbor 4X, y axis only:nny4x" );
                output_file.println ( "Option::Checkbox::-pce.stretch::Stretch to fill screen when in fullscreen mode.::0" );
                
                output_file.println ( "Option::Separator" );
                output_file.println ( "Option::Title::NEC PC-FX" );
                output_file.println ( "Option::Separator" );
                output_file.println ( "Option::Combobox::-pcfx.pixshader::Enable specified OpenGL pixel shader.::None::None:none:Auto Interpolation:autoip:Sharper Auto Interpolation:autoipsharper:Scale2x:scale2x:Sharper bilinear interpolation.:ipsharper:Linear interpolation on X axis only.:ipxnoty:Linear interpolation on Y axis only.:ipynotx:Sharper version of ipxnoty.:ipxnotysharper:Sharper version of ipynotx.:ipynotxsharper" );
                output_file.println ( "Option::Slider::-pcfx.scanlines::Enable scanlines with specified opacity.::0::0:100" );
                output_file.println ( "Option::Combobox::-pcfx.special::Enable specified special video scaler.::None::None:none:HQ2X Layer:hq2x:HQ3X:hq3x:HQ4X:hq4x:Scale 2X:scale2x:Scale 3X:scale3x:Scale 4X:scale4x:2XSAI:2xsai:Super 2XSAI:super2xsai:Super Eagle:supereagle:Nearest-neighbor 2X:nn2x:Nearest-Neighbor 3X:nn3x:Nearest-neighbor 4X:nn4x:Nearest-neighbor 2X, y axis only:nny2x:Nearest-neighbor 3X, y axis only:nny3x:Nearest-neighbor 4X, y axis only:nny4x" );
                output_file.println ( "Option::Checkbox::-pcfx.stretch::Stretch to fill screen when in fullscreen mode.::0" );
                
                output_file.println ( "Option::Separator" );
                output_file.println ( "Option::Title::Bandai Wonder Swan" );
                output_file.println ( "Option::Separator" );
                output_file.println ( "Option::Combobox::-wswan.pixshader::Enable specified OpenGL pixel shader.::None::None:none:Auto Interpolation:autoip:Sharper Auto Interpolation:autoipsharper:Scale2x:scale2x:Sharper bilinear interpolation.:ipsharper:Linear interpolation on X axis only.:ipxnoty:Linear interpolation on Y axis only.:ipynotx:Sharper version of ipxnoty.:ipxnotysharper:Sharper version of ipynotx.:ipynotxsharper" );
                output_file.println ( "Option::Slider::-wswan.scanlines::Enable scanlines with specified opacity.::0::0:100" );
                output_file.println ( "Option::Combobox::-wswan.special::Enable specified special video scaler.::None::None:none:HQ2X Layer:hq2x:HQ3X:hq3x:HQ4X:hq4x:Scale 2X:scale2x:Scale 3X:scale3x:Scale 4X:scale4x:2XSAI:2xsai:Super 2XSAI:super2xsai:Super Eagle:supereagle:Nearest-neighbor 2X:nn2x:Nearest-Neighbor 3X:nn3x:Nearest-neighbor 4X:nn4x:Nearest-neighbor 2X, y axis only:nny2x:Nearest-neighbor 3X, y axis only:nny3x:Nearest-neighbor 4X, y axis only:nny4x" );
                output_file.println ( "Option::Checkbox::-wswan.stretch::Stretch to fill screen when in fullscreen mode.::0" );
                
                output_file.println ( "Option::Separator" );
                output_file.println ( "Option::Title::Atari Lynx" );
                output_file.println ( "Option::Separator" );
                output_file.println ( "Option::Combobox::-lynx.pixshader::Enable specified OpenGL pixel shader.::None::None:none:Auto Interpolation:autoip:Sharper Auto Interpolation:autoipsharper:Scale2x:scale2x:Sharper bilinear interpolation.:ipsharper:Linear interpolation on X axis only.:ipxnoty:Linear interpolation on Y axis only.:ipynotx:Sharper version of ipxnoty.:ipxnotysharper:Sharper version of ipynotx.:ipynotxsharper" );
                output_file.println ( "Option::Slider::-lynx.scanlines::Enable scanlines with specified opacity.::0::0:100" );
                output_file.println ( "Option::Combobox::-lynx.special::Enable specified special video scaler.::None::None:none:HQ2X Layer:hq2x:HQ3X:hq3x:HQ4X:hq4x:Scale 2X:scale2x:Scale 3X:scale3x:Scale 4X:scale4x:2XSAI:2xsai:Super 2XSAI:super2xsai:Super Eagle:supereagle:Nearest-neighbor 2X:nn2x:Nearest-Neighbor 3X:nn3x:Nearest-neighbor 4X:nn4x:Nearest-neighbor 2X, y axis only:nny2x:Nearest-neighbor 3X, y axis only:nny3x:Nearest-neighbor 4X, y axis only:nny4x" );
                output_file.println ( "Option::Checkbox::-lynx.stretch::Stretch to fill screen when in fullscreen mode.::0" );
                break;
        }
        
        output_file.close ();
        
        return ( true );
    }
    
    public static boolean makeAudioPlugin ( String folder_path , int mednafen_version )
    {
        PrintWriter output_file;
        
        File file = new File ( folder_path );
        
        if ( file.isFile () )
        {
            if ( !file.delete () )
            {
                JOptionPane.showMessageDialog ( null , "Error deleing old plugin file on: " + folder_path + "." , "Error creating plugin file.", JOptionPane.ERROR_MESSAGE );
                return ( false );
            }
        }
        
        try
        {
            output_file = new PrintWriter ( new FileWriter ( folder_path + getAudioPluginName ( mednafen_version ) ) );
        }
        catch ( IOException e )
        {
            JOptionPane.showMessageDialog ( null , "Error creating plugin file: " + getAudioPluginName ( mednafen_version ) + "." , "Error creating plugin file", JOptionPane.ERROR_MESSAGE );
            return ( false );
        }
        
        printPreamble ( output_file );
        
        switch ( mednafen_version )
        {
            case MEDNAFEN_0_9_21:
                output_file.println ( "Option::Title::Generic options (all systems)" );
                output_file.println ( "Option::Separator" );
                output_file.println ( "Option::Checkbox::-sound::Enable sound output.::1" );
                if ( File.separator.equals ( "/" ) )
                {
                    output_file.println ( "Option::Slider::-sound.buffer_time::Desired total buffer size in milliseconds.::32::1:1000" );
                }
                else
                {
                    output_file.println ( "Option::Slider::-sound.buffer_time::Desired total buffer size in milliseconds.::52::1:1000" );
                }
                output_file.println ( "Option::Textfield::-sound.device::Select sound output device.::default" );
                output_file.println ( "Option::Combobox::-sound.driver::Select sound driver.::Default::Default:default:ALSA:alsa:Open Sound System:oss:DirectSound:dsound:Simple Directedia Layer (SDL):sdl:JACK:jack" );
                output_file.println ( "Option::Slider::-sound.rate::Specifies the sound playback rate, in frames per second (Hz).::48000::22050:1048576" );
                output_file.println ( "Option::Slider::-sound.volume::Sound volume level, in percent.::100::0:150" );
                output_file.println ( "Option::Checkbox::-ffnosound::Silence sound output when fast-forwarding.::0" );
                
                output_file.println ( "Option::Separator" );
                output_file.println ( "Option::Title::Sony PlayStation" );
                output_file.println ( "Option::Separator" );
                output_file.println ( "Option::Checkbox::-psx.forcemono::Force monophonic sound output.::0" );
                
                output_file.println ( "Option::Separator" );
                output_file.println ( "Option::Title::Nintendo Game Boy (Color)" );
                output_file.println ( "Option::Separator" );
                output_file.println ( "Option::Checkbox::-gb.forcemono::Force monophonic sound output.::0" );
                
                output_file.println ( "Option::Separator" );
                output_file.println ( "Option::Title::Nintendo Game Boy Advance" );
                output_file.println ( "Option::Separator" );
                output_file.println ( "Option::Checkbox::-gba.forcemono::Force monophonic sound output.::0" );
                
                output_file.println ( "Option::Separator" );
                output_file.println ( "Option::Title::Nintendo Virtual Boy" );
                output_file.println ( "Option::Separator" );
                output_file.println ( "Option::Checkbox::-vb.forcemono::Force monophonic sound output.::0" );
                
                output_file.println ( "Option::Separator" );
                output_file.println ( "Option::Title::Super Nintendo Entertainment System/Super Famicom" );
                output_file.println ( "Option::Separator" );
                output_file.println ( "Option::Checkbox::-snes.forcemono::Force monophonic sound output.::0" );
                
                output_file.println ( "Option::Separator" );
                output_file.println ( "Option::Title::Sega Game Gear" );
                output_file.println ( "Option::Separator" );
                output_file.println ( "Option::Checkbox::-gg.forcemono::Force monophonic sound output.::0" );
                
                output_file.println ( "Option::Separator" );
                output_file.println ( "Option::Title::Sega Genesis/MegaDrive" );
                output_file.println ( "Option::Separator" );
                output_file.println ( "Option::Checkbox::-md.forcemono::Force monophonic sound output.::0" );
                
                output_file.println ( "Option::Separator" );
                output_file.println ( "Option::Title::Sega Master System" );
                output_file.println ( "Option::Separator" );
                output_file.println ( "Option::Checkbox::-sms.forcemono::Force monophonic sound output.::0" );
                
                output_file.println ( "Option::Separator" );
                output_file.println ( "Option::Title::SNK Neo Geo Pocket (Color)" );
                output_file.println ( "Option::Separator" );
                output_file.println ( "Option::Checkbox::-ngp.forcemono::Force monophonic sound output.::0" );
                
                output_file.println ( "Option::Separator" );
                output_file.println ( "Option::Title::NEC PC Engine (CD)/TurboGrafx 16 (CD)/SuperGrafx" );
                output_file.println ( "Option::Separator" );
                output_file.println ( "Option::Checkbox::-pce.forcemono::Force monophonic sound output.::0" );
                
                output_file.println ( "Option::Separator" );
                output_file.println ( "Option::Title::PC Engine (CD)/TurboGrafx 16 (CD)/SuperGrafx -> Fast emulation" );
                output_file.println ( "Option::Separator" );
                output_file.println ( "Option::Checkbox::-pce_fast.forcemono::Force monophonic sound output.::0" );
                
                output_file.println ( "Option::Separator" );
                output_file.println ( "Option::Title::NEC PC-FX" );
                output_file.println ( "Option::Separator" );
                output_file.println ( "Option::Checkbox::-pcfx.forcemono::Force monophonic sound output.::0" );
                
                output_file.println ( "Option::Separator" );
                output_file.println ( "Option::Title::Bandai Wonder Swan" );
                output_file.println ( "Option::Separator" );
                output_file.println ( "Option::Checkbox::-wswan.forcemono::Force monophonic sound output.::0" );
                break;
            case MEDNAFEN_0_8_D:
                output_file.println ( "Option::Title::Generic options (all systems)" );
                output_file.println ( "Option::Separator" );
                output_file.println ( "Option::Checkbox::-sound::Enable sound output.::1" );
                if ( File.separator.equals ( "/" ) )
                {
                    output_file.println ( "Option::Slider::-soundbufsize::Desired total buffer size in milliseconds.::32::1:1000" );
                }
                else
                {
                    output_file.println ( "Option::Slider::-soundbufsize::Desired total buffer size in milliseconds.::52::1:1000" );
                }
                output_file.println ( "Option::Textfield::-sounddevice::Select sound output device.::default" );
                output_file.println ( "Option::Combobox::-sounddriver::Select sound driver.::Default::Default:default:ALSA:alsa:Open Sound System:oss:DirectSound:dsound:Simple Directedia Layer (SDL):sdl:JACK:jack" );
                output_file.println ( "Option::Slider::-soundrate::Specifies the sound playback rate, in frames per second (Hz).::48000::8192:48000" );
                output_file.println ( "Option::Slider::-soundvol::Sound volume level, in percent.::100::0:100" );
                output_file.println ( "Option::Checkbox::-ffnosound::Silence sound output when fast-forwarding.::0" );
                
                output_file.println ( "Option::Separator" );
                output_file.println ( "Option::Title::Nintendo Game Boy (Color)" );
                output_file.println ( "Option::Separator" );
                output_file.println ( "Option::Checkbox::-gb.forcemono::Force monophonic sound output.::0" );
                
                output_file.println ( "Option::Separator" );
                output_file.println ( "Option::Title::Nintendo Game Boy Advance" );
                output_file.println ( "Option::Separator" );
                output_file.println ( "Option::Checkbox::-gba.forcemono::Force monophonic sound output.::0" );
                
                output_file.println ( "Option::Separator" );
                output_file.println ( "Option::Title::SNK Neo Geo Pocket (Color)" );
                output_file.println ( "Option::Separator" );
                output_file.println ( "Option::Checkbox::-ngp.forcemono::Force monophonic sound output.::0" );
                
                output_file.println ( "Option::Separator" );
                output_file.println ( "Option::Title::NEC PC Engine (CD)/TurboGrafx 16 (CD)/SuperGrafx" );
                output_file.println ( "Option::Separator" );
                output_file.println ( "Option::Checkbox::-pce.forcemono::Force monophonic sound output.::0" );
                break;
        }
        
        output_file.close ();
        
        return ( true );
    }
    
    public static boolean makeControlsPlugin ( String folder_path , int mednafen_version )
    {
        PrintWriter output_file;
        
        File file = new File ( folder_path );
        
        if ( file.isFile () )
        {
            if ( !file.delete () )
            {
                JOptionPane.showMessageDialog ( null , "Error deleing old plugin file on: " + folder_path + "." , "Error creating plugin file.", JOptionPane.ERROR_MESSAGE );
                return ( false );
            }
        }
        
        try
        {
            output_file = new PrintWriter ( new FileWriter ( folder_path + getControlsPluginName ( mednafen_version ) ) );
        }
        catch ( IOException e )
        {
            JOptionPane.showMessageDialog ( null , "Error creating plugin file: " + getControlsPluginName ( mednafen_version ) + "." , "Error creating plugin file", JOptionPane.ERROR_MESSAGE );
            return ( false );
        }
        
        printPreamble ( output_file );
        
        switch ( mednafen_version )
        {
            case MEDNAFEN_0_9_21:
                output_file.println ( "Option::Title::Generic options" );
                output_file.println ( "Option::Separator" );
                output_file.println ( "Option::Slider::-analogthreshold::Analog axis press threshold.::75::0:100" );
                output_file.println ( "Option::Slider::-autofirefreq::Auto-fire frequency.::3::0:1000" );
                break;
            case MEDNAFEN_0_8_D:
                output_file.println ( "Option::Title::Generic options" );
                output_file.println ( "Option::Separator" );
                output_file.println ( "Option::Slider::-analogthreshold::Analog axis press threshold.::75::0:100" );
                output_file.println ( "Option::Slider::-autofirefreq::Auto-fire frequency.::2::0:1000" );
                break;
        }
        
        output_file.close ();
        
        return ( true );
    }
    
    public static boolean makeNetworkPlugin ( String folder_path , int mednafen_version )
    {
        PrintWriter output_file;
        
        File file = new File ( folder_path );
        
        if ( file.isFile () )
        {
            if ( !file.delete () )
            {
                JOptionPane.showMessageDialog ( null , "Error deleing old plugin file on: " + folder_path + "." , "Error creating plugin file.", JOptionPane.ERROR_MESSAGE );
                return ( false );
            }
        }
        
        try
        {
            output_file = new PrintWriter ( new FileWriter ( folder_path + getNetworkPluginName ( mednafen_version ) ) );
        }
        catch ( IOException e )
        {
            JOptionPane.showMessageDialog ( null , "Error creating plugin file: " + getNetworkPluginName ( mednafen_version ) + "." , "Error creating plugin file", JOptionPane.ERROR_MESSAGE );
            return ( false );
        }
        
        printPreamble ( output_file );
        
        switch ( mednafen_version )
        {
            case MEDNAFEN_0_9_21:
                output_file.println ( "Option::Title::Generic options" );
                output_file.println ( "Option::Separator" );
                output_file.println ( "Option::ConditionalCheckbox::-connect::Trigger to connect to remote host after the game is loaded.::0" );
                output_file.println ( "Option::Textfield::-netplay.gamekey::Key to hash with the MD5 hash of the game.::0" );
                output_file.println ( "Option::Textfield::-netplay.host::Server hostname.::fobby.net" );
                output_file.println ( "Option::Slider::-netplay.localplayers::Local player count.::1::1:8" );
                output_file.println ( "Option::Textfield::-netplay.nick::Nickname.::JMednafenPlayer" );
                output_file.println ( "Option::Slider::-netplay.port::Server port.::4046::1:65535" );
                output_file.println ( "Option::Checkbox::-netplay.smallfont::Use small (tiny!) font for netplay chat console.::0" );
                break;
            case MEDNAFEN_0_8_D:
                output_file.println ( "Option::Title::Generic options" );
                output_file.println ( "Option::Separator" );
                output_file.println ( "Option::ConditionalCheckbox::-connect::Trigger to connect to remote host after the game is loaded.::0" );
                output_file.println ( "Option::Textfield::-nethost::Server hostname.::fobby.net" );
                output_file.println ( "Option::Slider::-netlocalplayers::Local player count.::1::1:8" );
                output_file.println ( "Option::Textfield::-netnick::Nickname.::JMednafenPlayer" );
                output_file.println ( "Option::Slider::-netport::Server port.::4046::1:65535" );
                break;
        }
        
        output_file.close ();
        
        return ( true );
    }
    
    public static boolean makeInGamePlugin ( String folder_path , int mednafen_version )
    {
        PrintWriter output_file;
        
        File file = new File ( folder_path );
        
        if ( file.isFile () )
        {
            if ( !file.delete () )
            {
                JOptionPane.showMessageDialog ( null , "Error deleing old plugin file on: " + folder_path + "." , "Error creating plugin file.", JOptionPane.ERROR_MESSAGE );
                return ( false );
            }
        }
        
        try
        {
            output_file = new PrintWriter ( new FileWriter ( folder_path + getInGamePluginName ( mednafen_version ) ) );
        }
        catch ( IOException e )
        {
            JOptionPane.showMessageDialog ( null , "Error creating plugin file: " + getInGamePluginName ( mednafen_version ) + "." , "Error creating plugin file", JOptionPane.ERROR_MESSAGE );
            return ( false );
        }
        
        printPreamble ( output_file );
        
        switch ( mednafen_version )
        {
            case MEDNAFEN_0_9_21:
                output_file.println ( "Option::Title::Generic options" );
                output_file.println ( "Option::Separator" );
                output_file.println ( "Option::Checkbox::-autosave::Automatic load/save state on game load/save.::0" );
                output_file.println ( "Option::Checkbox::-cheats::Enable cheats.::1" );
                output_file.println ( "Option::Slider::-ffspeed::Fast-forwarding speed multiplier::4::1:15" );
                output_file.println ( "Option::Checkbox::-fftoggle::Treat the fast-forward button as a toggle.::0" );
                output_file.println ( "Option::Checkbox::-filesys.disablesavegz::Disable gzip compression when saving save states and backup memory.::0" );
                output_file.println ( "Option::Checkbox::-nothrottle::Disable speed throttling when sound is disabled.::0" );
                output_file.println ( "Option::Slider::-osd.state_display_time::Time, in milliseconds, to display the save state or the movie selector.::2000::0:15000" );
                break;
            case MEDNAFEN_0_8_D:
                output_file.println ( "Option::Title::Generic options" );
                output_file.println ( "Option::Separator" );
                output_file.println ( "Option::Checkbox::-autosave::Automatic load/save state on game load/save.::0" );
                output_file.println ( "Option::Slider::-ffspeed::Fast-forwarding speed multiplier::2::1:10" );
                output_file.println ( "Option::Checkbox::-fftoggle::Treat the fast-forward button as a toggle.::0" );
                break;
        }
        
        output_file.close ();
        
        return ( true );
    }
    
    public static boolean makeOtherPlugin ( String folder_path , int mednafen_version )
    {
        PrintWriter output_file;
        
        File file = new File ( folder_path );
        
        if ( file.isFile () )
        {
            if ( !file.delete () )
            {
                JOptionPane.showMessageDialog ( null , "Error deleing old plugin file on: " + folder_path + "." , "Error creating plugin file.", JOptionPane.ERROR_MESSAGE );
                return ( false );
            }
        }
        
        try
        {
            output_file = new PrintWriter ( new FileWriter ( folder_path + getOtherPluginName ( mednafen_version ) ) );
        }
        catch ( IOException e )
        {
            JOptionPane.showMessageDialog ( null , "Error creating plugin file: " + getOtherPluginName ( mednafen_version ) + "." , "Error creating plugin file", JOptionPane.ERROR_MESSAGE );
            return ( false );
        }
        
        printPreamble ( output_file );
        
        switch ( mednafen_version )
        {
            case MEDNAFEN_0_9_21:
                output_file.println ( "Option::Title::Generic options" );
                output_file.println ( "Option::Separator" );
                output_file.println ( "Option::Combobox::-srwcompressor::Compressor to use with state rewinding::QuickLZ::MiniLZO:minilzo:QuickLZ:quicklz:BLZ:blz" );
                output_file.println ( "Option::Slider::-srwframes::Number of frames to keep states for when state rewinding is enabled.::600::10:99999" );
                break;
            case MEDNAFEN_0_8_D:
                output_file.println ( "Option::Title::Generic options" );
                output_file.println ( "Option::Separator" );
                output_file.println ( "Option::Combobox::-srwcompressor::Compressor to use with state rewinding::QuickLZ::MiniLZO:minilzo:QuickLZ:quicklz:BLZ:blz" );
                output_file.println ( "Option::Slider::-srwframes::Number of frames to keep states for when state rewinding is enabled.::600::10:99999" );
                break;
        }
        
        output_file.close ();
        
        return ( true );
    }
}
